// 
// tiny example client to the game to illustrate the protocol.
// use it to decode the messages.
//

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>

#include <yarp/os/all.h>

using namespace yarp::os;

// a broadcast message (result of a say) arrives via port as a Bottle containing a string: e.g.
// char *broadcast = bottle.toString().c_str();


// see code.
void look(Port& p) {

    Bottle send("look");
    Property prop;

    // send look, gets a sort of Bottle back.
    p.write(send, prop);
    printf ("%s\n", prop.toString().c_str());

    printf ("\n\n");
    // otherwise.
    
    // find a group that contains the map.
    //
    Bottle& map = prop.findGroup("look").findGroup("map");
    int i;
    for (i = 1; i < map.size(); i++) {
        printf("  %s\n", map.get(i).asString().c_str());
    }
    printf("\n");
    
    // find the list of other players (other connected clients).
    //
    Bottle& players = prop.findGroup("look").findGroup("players");

    // print other player information.
    //
    for (i = 1; i < players.size(); i++) {
        Bottle *player = players.get(i).asList();
        if (player != NULL) {
            Bottle& location = player->findGroup("location");
            Value& life = player->find("life");
            ConstString playerName = player->get(0).asString();
            if (strlen(playerName.c_str()) < 40) {
                printf("PLAYER %s is at (%d,%d) with lifeforce %d\n\n", 
                                    playerName.c_str(), 
                                    location.get(1).asInt(),
                                    location.get(2).asInt(),
                                    life.asInt());
            }
        }
    }
}

// move and one of (left, right, up, down)
void move(Port& p) {
    Bottle send("go left"), recv;
    p.write(send, recv);
    if (recv.get(0).asString() == "error") {
        printf("PROBLEM:\n");
        printf("  request: %s\n", send.toString().c_str());
        printf("  response: %s\n", recv.toString().c_str());
    }
}

// fire and one of (left, right, up, down)
void fire(Port& p) {
    Bottle send("fire down"), recv;
    p.write(send, recv);
    if (recv.get(0).asString() == "error") {
        printf("PROBLEM:\n");
        printf("  request: %s\n", send.toString().c_str());
        printf("  response: %s\n", recv.toString().c_str());
    }
}

// say + a generic text
void say(Port& p) {
    Bottle send("say my message here"), recv;
    p.write(send, recv);
    if (recv.get(0).asString() == "error") {
        printf("PROBLEM:\n");
        printf("  request: %s\n", send.toString().c_str());
        printf("  response: %s\n", recv.toString().c_str());
    }
}

//
//
//
int main(int argc, char *argv[]) {
    Network::init();

    Port p;
    p.open("/player/test");

    printf("Connecting...\n");
    // connecting.
    // connect our port to the game server.
    Network::connect(p.getName(),"/game");
    // connect the game back to us using the Multicast protocol.
    // this requires another thread to be reading on the port and
    // get the message when it arrives.
    Network::connect("/game",p.getName(),"mcast");

    printf("Hopefully connected\n");

    Time::delay(2);
    printf("Ask for the view of the game\n\n");
    look(p);

    Time::delay(2);
    printf("Asking a move\n");
    move(p);

    Time::delay(2);
    printf("Fire somewhere\n");
    fire(p);

    Time::delay(2);
    printf("Say something\n");
    say(p);

    Time::delay(2);
    printf("Close port\n");
    p.close();

    Network::fini();
    return 0;
}

